--[[
	GUI Elements for Cargolist
]]

cargolist.ui = {}
cargolist.ui.font = 14 * (gkinterface.GetYResolution () / 600)
cargolist.ui.fontSmall = 10 * (gkinterface.GetYResolution () / 600)
cargolist.ui.fgcolor = "200 200 50"
cargolist.ui.fgcolor_unselected = "200 200 200 75 *"
cargolist.ui.fgcolor_selected = "200 200 50 150 *"
cargolist.ui.bgcolor_unselected = "255 10 10 10 *"
cargolist.ui.bgcolor_selected = "255 150 150 150 *"
dofile ("ui/control.lua")
cargolist.ui.sortColumn = 1
cargolist.ui.controlWidth = 60
dofile ("ui/hud.lua")

function cargolist.ui:GetOnOffSetting (flag)
	if (flag) then
		return "ON"
	else
		return "OFF"
	end
end

local isShowing = false
function cargolist.ui:CreateUI ()
	-- Check if this list is already being displayed
	if isShowing then
		ProcessEvent ("CARGOLIST_RELOAD")
		return
	end
	
	local refreshButton = iup.stationbutton { title="Refresh", font=cargolist.ui.font}
	local closeButton = iup.stationbutton { title="Close", font=cargolist.ui.font}
	
	-- Active Set Selection
	local activeSetSelection = iup.pdasublist  {
		font = cargolist.ui.font,
		dropdown = "YES",
		visible_items = 10,
		expand = "HORIZONTAL"
	}
	activeSetSelection [1] = "All"
	local set
	local index = 2
	for set,_ in pairs (cargolist.data.cargoSets) do
		activeSetSelection [index] = set
		if set == cargolist.data.activeSet then
			activeSetSelection.value = index
		end
		index = index + 1
	end
	
	function activeSetSelection:action (text, index, state)
		if state == 1 then
			cargolist.util:SetActiveSet (text)
			cargolist.data:SaveOptions ()
		end
	end
	
	-- Build Cargo Display Matrix
	local l
	local matrix = iup.pdasubsubmatrix {
		numcol = 4,
		numlin = 0,
		numlin_visible = 10,
		heightdef = 15,
		expand = "YES",
		font = cargolist.ui.font,
		bgcolor = cargolist.ui.bgcolor_unselected,
		redraw = "ALL",
		width1 = 250,
		width2 = 100,
		width3 = 0,
		width4 = 0
	}
	
	-- Set Headers
	matrix:setcell (0, 1, "Cargo")
	matrix:setcell (0, 2, "Distance")
	
	local pda = iup.pdarootframe {
		iup.vbox {
			iup.hbox {
				iup.fill {size=5},
				iup.label {title="Set:", font=cargolist.ui.font, fgcolor=cargolist.ui.fgcolor},
				iup.fill {size=10},
				activeSetSelection;
			},
			matrix,
			iup.fill {},
			iup.hbox {
				iup.fill {},
				refreshButton,
				closeButton; };
		};
	}
	local frame = iup.dialog {
		pda,
	    title = "Cargo Scan",
	    font = cargolist.ui.font,
		border = 'YES',
		topmost = 'YES',
		resize = 'NO',
		maxbox = 'NO',
		minbox = 'NO',
		modal = 'NO',
		fullscreen = 'NO',
		expand = 'YES',
		active = 'NO',
		menubox = 'NO',
		bgcolor = cargolist.ui.bgcolor_unselected,
		defaultenter = refreshButton,
		defaultesc = closeButton
	}
	
	local function SetSelectedMatrixRow (row)
		for l=1,matrix.numlin do
			if l == row then
				matrix ["bgcolor"..l..":*"] = cargolist.ui.bgcolor_selected
			else
				matrix ["bgcolor"..l..":*"] = cargolist.ui.bgcolor_unselected
			end
		end
	end
	
	local function ClearData ()
		local i
		for i=1, tonumber (matrix.numlin) do
			matrix.dellin = 1
		end
	end
	
	local function ReloadData (doSort)
		if doSort == nil then doSort = true end
		local list = cargolist.data.itemList
		if doSort then
			cargolist.util:SortDrops (list)
		end
		ClearData ()
		local row = 0
		matrix.heightdef = 15
		matrix.redraw = "ALL"
		local item, v
		for _,v in ipairs (list) do
			matrix.addlin = row
			matrix.font = cargolist.ui.font
			row = row + 1
			matrix:setcell (row, 1, tostring (v.name))
			matrix:setcell (row, 2, tostring (math.floor (v.dist)) .. "m  ")
			matrix:setcell (row, 3, tostring (v.id))
			matrix:setcell (row, 4, tostring (v.type))
			matrix.alignment1 = "ALEFT"
			matrix.alignment2 = "ARIGHT"
			matrix.width1 = 250
			matrix.width2 = 100
			matrix.width3 = 0
			matrix.width4 = 0
			cargolist:Yield ()
		end
		matrix.numlin = row
		if #list > 0 then
			SetSelectedMatrixRow (cargolist.data.currentDrop)
		end
		iup.Refresh (frame)
	end
	
	matrix.click_cb = function (self, row, col)
		-- Set all bgcolors
		cargolist.data.currentDrop = row
		SetSelectedMatrixRow (row)
		
		if row == 0 then
			cargolist.ui.sortColumn = col
			ReloadData ()
		else
			-- Set Target to row element
			local id = cargolist.data.itemList [row].id
			local type = cargolist.data.itemList [row].type
			radar.SetRadarSelection (type, id)
		end
	end
	
	refreshButton.action = function ()
		-- Scan for all cargo and reload matrix
		if not cargolist.data.scanner then
			cargolist.data.scanner = coroutine.create (function ()
				print ("\12700ff00Scanning...\127o")
				cargolist.util:ScanCargo ()
				ReloadData ()
				print ("\12700ff00Done\127o")
				cargolist.util:SetLocks (false)
				cargolist:KillThread ()
			end)
			return cargolist:RunScan ()
		end
	end

	closeButton.action = function ()
		UnregisterEvent (frame, "CARGOLIST_RELOAD")
		UnregisterEvent (frame, "CARGOLIST_SET")
		UnregisterEvent (frame, "SECTOR_CHANGED")
		UnregisterEvent (frame, "ENTERED_STATION")
		HideDialog (frame)
		frame.active = "NO"
		gkini.WriteInt (cargolist.config, "showXPos", frame.x)
		gkini.WriteInt (cargolist.config, "showYPos", frame.y)
		cargolist.data.scanner = nil
		cargolist.util:SetLocks (false)
		isShowing = false
		ProcessEvent ("CARGOLIST_RELOAD")
	end
	
	local function DoUpdate ()
		if isShowing then
			Timer ():SetTimeout (1000, function ()
				local row, item
				cargolist.util:SetLocks (true)
				local objectType, objectId = radar.GetRadarSelectionID ()
				gkinterface.GKProcessCommand ("RadarNone")
				local removeLost = false
				for row, item in ipairs (cargolist.data.itemList) do
					if item:Update () then
						matrix:setcell (row, 2, tostring (math.floor (item.dist)) .. "m  ")
					else
						removeLost = true
						table.remove (cargolist.data.itemList, row)
						if cargolist.data.currentDrop > 1 and row <= cargolist.data.currentDrop then
							cargolist.data.currentDrop = cargolist.data.currentDrop - 1
						end
					end
				end
				radar.SetRadarSelection (objectType, objectId)
				if removeLost then
					ReloadData (false)
					cargolist.util:SetTarget ()
				end
				cargolist.util:SetLocks (false)
				DoUpdate ()
			end)
		end
	end
	
	-- Events
	function frame:OnEvent (event, data)
		if event == "CARGOLIST_RELOAD" then
			ReloadData ()
			
		elseif event == "CARGOLIST_SET" then
			cargolist.data.currentDrop = data
			SetSelectedMatrixRow (data)
			
		elseif event == "SECTOR_CHANGED" then
			DoUpdate ()
			
		elseif event == "ENTERED_STATION" then
			DoUpdate ()	
		end
	end
	
	-- Display dialog at stored coordinates
	local x = gkini.ReadInt (cargolist.config, "showXPos", iup.CENTER)
	local y = gkini.ReadInt (cargolist.config, "showYPos", iup.CENTER)
	ShowDialog (frame, x, y)
	ReloadData ()
	frame.active = "YES"
	isShowing = true
	RegisterEvent (frame, "CARGOLIST_RELOAD")
	RegisterEvent (frame, "CARGOLIST_SET")
	RegisterEvent (frame, "SECTOR_CHANGED")
	RegisterEvent (frame, "ENTERED_STATION")
	DoUpdate ()
	
	return frame
end

function cargolist.ui:CreateAlertUI (msg)
	local okButton = iup.stationbutton {title="Ok", focus="YES"}
	
	local pda = iup.pdasubsubframebg {
		iup.hbox {
			iup.fill {size=10},
			iup.vbox {
				iup.fill {size=5},
				iup.label {title=msg, fgcolor=cargolist.ui.fgcolor, alignment="ACENTER", expand="HORIZONTAL"},
				iup.fill {size=15},
				iup.hbox {
					iup.fill {},
					okButton;
					expand="HORIZONTAL"
				},
				iup.fill {size=5};
				expand="YES"
			},
			iup.fill {size=10};
			expand="YES"
		};
		expand="YES"
	}
	
	local frame = iup.dialog {
		pda,
	    font = cargolist.ui.font,
		border = 'YES',
		topmost = 'YES',
		resize = 'YES',
		maxbox = 'NO',
		minbox = 'NO',
		modal = 'NO',
		fullscreen = 'NO',
		expand = "YES",
		active = 'YES',
		menubox = 'NO',
		bgcolor = cargolist.ui.bgcolor_unselected,
		defaultesc = okButton,
		defaultenter = okButton
	}
	
	okButton.action = function ()
		HideDialog (frame)
		frame.active = "NO"
	end
	
	ShowDialog (frame, iup.CENTER, iup.CENTER)
	frame.active = "YES"
	
	return frame
end